@@ -1,5 +1,4 @@
 #!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 
 from .fonts import *
 from trojanzoo.utils import to_numpy
@@ -37,8 +36,6 @@ class Figure:
         self.folder_path: str = folder_path
         if folder_path is None:
             self.folder_path = './output/'
-        if not os.path.exists(self.folder_path):
-            os.makedirs(self.folder_path)
         self.fig: Figure = fig
         self.ax: Axes = ax
         if fig is None and ax is None:
@@ -64,16 +61,16 @@ class Figure:
         getattr(self.ax, f'set_{axis}label')(text, fontsize=fontsize,
                                              family=family, font=font, weight=weight, **kwargs)
 
-    def set_title(self, text: str = None, fontsize: int = 16) -> None:
+    def set_title(self, text: str = None, fontsize: int = 16, fontweight: str = 'bold') -> None:
         if text is None:
             text = self.name
-        self.ax.set_title(text, fontsize=fontsize)
+        self.ax.set_title(text, fontsize=fontsize, fontweight=fontweight)
 
-    def save(self, path: str = None, folder_path: str = None, ext: str = 'pdf') -> None:
+    def save(self, path: str = None, folder_path: str = None, ext: str = '.pdf') -> None:
         if path is None:
             if folder_path is None:
                 folder_path = self.folder_path
-            path = f'{folder_path}{self.name}.{ext}'
+            path = os.path.join(folder_path, f'{self.name}{ext}')
         else:
             folder_path = os.path.dirname(path)
         if not os.path.exists(folder_path):
@@ -125,10 +122,6 @@ class Figure:
             self.curve_legend(label=label, color=color, linewidth=linewidth, marker=marker, **kwargs)
         return self.ax.scatter(x, y, color=color, linewidth=linewidth, marker=marker, facecolor=facecolor, zorder=zorder, **kwargs)
 
-    def add_subplot(self, projection=None):
-        if projection is not None:
-            return self.fig.add_subplot(projection=projection)
-
 # Markers
 # '.' point marker
 # ',' pixel marker
@@ -189,8 +182,7 @@ class Figure:
                              ha='center', va='bottom', fontsize=fontsize)
 
     @staticmethod
-    def get_roc_curve(label, pred) -> tuple[list[float], list[float]]:
-
+    def get_roc_curve(label: list[int], pred: list[int]) -> tuple[list[float], list[float]]:
         total_inst = len(label)
         total_pos_inst = len(np.where(label == 1)[0])
 
@@ -200,28 +192,19 @@ class Figure:
 
         # iterate over all positive thresholds
         for threshold in np.unique(pred):
-
-            pred_pos_idx = np.where(pred >= threshold)[0]
-
+            pred_pos_idx: np.ndarray = np.where(pred >= threshold)[0]
             # number of predicted positive instances
             pred_pos_inst = len(pred_pos_idx)
             # number of true positive instances
-            true_pos_inst = np.count_nonzero(label[pred_pos_idx])
+            true_pos_inst: int = np.count_nonzero(label[pred_pos_idx])
 
-            tpr = true_pos_inst * 1. / total_pos_inst * 1.
-            fpr = (pred_pos_inst - true_pos_inst) * \
-                1. / (total_inst - total_pos_inst) * 1.
+            tpr = true_pos_inst / total_pos_inst
+            fpr = (pred_pos_inst - true_pos_inst) / (total_inst - total_pos_inst)
             tprs.append(tpr)
             fprs.append(fpr)
             thresholds.append(threshold)
-
         return fprs, tprs
 
-    @staticmethod
-    def sort(x: np.ndarray, y: np.ndarray) -> tuple[np.ndarray, np.ndarray]:
-        idx = np.argsort(x)
-        return np.array(x)[idx], np.array(y)[idx]
-
     @staticmethod
     def normalize(x: np.ndarray, _min: float = None, _max: float = None, tgt_min: float = 0.0, tgt_max: float = 1.0) -> np.ndarray:
         x = to_numpy(x)
