@@ -1,5 +1,4 @@
 #!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 
 from .imagefolder import ImageFolder
 import torchvision.transforms as transforms
@@ -9,10 +8,11 @@ import numpy as np
 class GTSRB(ImageFolder):
 
     name = 'gtsrb'
-    n_dim = (32, 32)
+    data_shape = [3, 32, 32]
     num_classes = 43
     valid_set = False
     url = {'train': 'https://sid.erda.dk/public/archives/daaeac0d7ce1152aea9b61d9f1e19370/GTSRB-Training_fixed.zip'}
+    md5 = {'train': '513f3c79a4c5141765e10e952eaa2478'}
     org_folder_name = {'train': 'GTSRB/Training'}
 
     def __init__(self, norm_par: dict[str, list[float]] = {'mean': [0.3403, 0.3121, 0.3214],
