@@ -1,5 +1,4 @@
 #!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 
 from trojanzoo.datasets import Dataset
 from trojanvision.environ import env
@@ -7,9 +6,13 @@ from trojanvision.environ import env
 import torch
 import torch.utils.data
 import torchvision.transforms as transforms
-from torchvision.datasets import VisionDataset
 import os
+
+from typing import TYPE_CHECKING
+from torchvision.datasets import VisionDataset  # TODO: python 3.10
 import PIL.Image as Image
+if TYPE_CHECKING:
+    pass
 
 
 class ImageSet(Dataset):
@@ -17,14 +20,13 @@ class ImageSet(Dataset):
     name: str = 'imageset'
     data_type: str = 'image'
     num_classes = 1000
-    n_channel: int = 3
-    n_dim = (224, 224)
+    data_shape = [3, 224, 224]
 
     def __init__(self, norm_par: dict[str, list[float]] = None,
                  default_model: str = 'resnetcomp18', **kwargs):
         super().__init__(default_model=default_model, **kwargs)
         self.norm_par: dict[str, list[float]] = norm_par
-        self.param_list['imageset'] = ['n_channel', 'n_dim', 'norm_par']
+        self.param_list['imageset'] = ['data_shape', 'norm_par']
 
     @staticmethod
     def get_transform(**kwargs) -> transforms.ToTensor:
@@ -65,7 +67,7 @@ class ImageSet(Dataset):
                 image: Image.Image
                 target_class: int
                 class_name = idx_to_class[target_class]
-                _dir = self.folder_path + self.name + f'/{mode}/{class_name}/'
+                _dir = os.path.join(self.folder_path, self.name, mode, class_name)
                 if not os.path.exists(_dir):
                     os.makedirs(_dir)
                 image.save(_dir + f'{class_counters[target_class]}{img_type}')
