@@ -1,5 +1,4 @@
 #!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 
 from trojanvision.optim import PGD
 from trojanzoo.defenses import Defense
@@ -23,7 +22,7 @@ class Grad_Train(Defense):
         self.pgd = PGD(alpha=pgd_alpha, epsilon=pgd_epsilon, iteration=pgd_iteration, stop_threshold=None)
 
     def detect(self, **kwargs):
-        self.model._train(loss_fn=self.loss_fn, validate_func=self.validate_func, verbose=True, **kwargs)
+        self.model._train(loss_fn=self.loss_fn, validate_fn=self.validate_fn, verbose=True, **kwargs)
 
     def loss_fn(self, _input, _label, **kwargs):
         new_input = _input.repeat(4, 1, 1, 1)
@@ -38,15 +37,16 @@ class Grad_Train(Defense):
         new_loss = loss + self.grad_lambda * grad.flatten(start_dim=1).norm(p=1, dim=1).mean()
         return new_loss
 
-    def validate_func(self, get_data_fn=None, loss_fn=None, **kwargs) -> tuple[float, float, float]:
-        clean_loss, clean_acc = self.model._validate(print_prefix='Validate Clean',
-                                                        get_data_fn=None, **kwargs)
-        adv_loss, adv_acc = self.model._validate(print_prefix='Validate Adv',
-                                                    get_data_fn=self.get_data, **kwargs)
+    def validate_fn(self, get_data_fn=None, loss_fn=None, **kwargs) -> tuple[float, float]:
+        # TODO
+        _, clean_acc = self.model._validate(print_prefix='Validate Clean',
+                                            get_data_fn=None, **kwargs)
+        _, adv_acc = self.model._validate(print_prefix='Validate Adv',
+                                          get_data_fn=self.get_data, **kwargs)
         # todo: Return value
         if self.clean_acc - clean_acc > 20 and self.clean_acc > 40:
             adv_acc = 0.0
-        return clean_loss + adv_loss, adv_acc, clean_acc
+        return clean_acc, adv_acc
 
     def get_data(self, data: tuple[torch.Tensor, torch.Tensor], **kwargs) -> tuple[torch.Tensor, torch.Tensor]:
         _input, _label = self.model.get_data(data, **kwargs)
