@@ -1,5 +1,4 @@
 #!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 
 from ..backdoor_defense import BackdoorDefense
 from trojanvision.environ import env
@@ -44,9 +43,6 @@ class ABS(BackdoorDefense):
                  samp_k: int = 1, same_range: bool = False, n_samples: int = 5,
                  max_troj_size: int = 16, remask_lr: float = 0.1, remask_epoch: int = 1000, **kwargs):
         super().__init__(**kwargs)
-        data_shape = [self.dataset.n_channel]
-        data_shape.extend(self.dataset.n_dim)
-        self.data_shape: list[int] = data_shape
 
         self.seed_num: int = seed_num
         if self.seed_num < 0:
@@ -151,10 +147,13 @@ class ABS(BackdoorDefense):
                 print(_str)
                 if not os.path.exists(self.folder_path):
                     os.makedirs(self.folder_path)
-                np.save(self.folder_path + self.get_filename(target_class=self.target_class) + '.npy', neuron_dict)
-                np.save(self.folder_path + self.get_filename(target_class=self.target_class) + '_best.npy', result_dict)
-            print(
-                f'Label: {label:3d}  loss: {result_dict[label]["loss"]:10.3f}  ATK loss: {result_dict[label]["attack_loss"]:10.3f}  Norm: {result_dict[label]["norm"]:10.3f}  Jaccard: {result_dict[label]["jaccard"]:10.3f}  Score: {best_score:.3f}')
+                np.save(os.path.join(self.folder_path,
+                                     self.get_filename(target_class=self.target_class) + '.npy'), neuron_dict)
+                np.save(os.path.join(self.folder_path,
+                                     self.get_filename(target_class=self.target_class) + '_best.npy'), result_dict)
+            print(f'Label: {label:3d}  loss: {result_dict[label]["loss"]:10.3f}'
+                  f'  ATK loss: {result_dict[label]["attack_loss"]:10.3f}  Norm: {result_dict[label]["norm"]:10.3f}'
+                  f'  Jaccard: {result_dict[label]["jaccard"]:10.3f}  Score: {best_score:.3f}')
             score_list[label] = best_score
         print('Score: ', score_list)
         print('Score MAD: ', normalize_mad(score_list))
@@ -163,11 +162,11 @@ class ABS(BackdoorDefense):
     def remask(self, _input: torch.Tensor, layer: str, neuron: int,
                label: int, use_mask: bool = True, validate_interval: int = 100,
                verbose=False) -> tuple[torch.Tensor, torch.Tensor, float]:
-        atanh_mark = torch.randn(self.data_shape, device=env['device'])
+        atanh_mark = torch.randn(self.dataset.data_shape, device=env['device'])
         atanh_mark.requires_grad_()
         parameters: list[torch.Tensor] = [atanh_mark]
-        mask = torch.ones(self.data_shape[1:], device=env['device'])
-        atanh_mask = torch.ones(self.data_shape[1:], device=env['device'])
+        mask = torch.ones(self.dataset.data_shape[1:], device=env['device'])
+        atanh_mask = torch.ones(self.dataset.data_shape[1:], device=env['device'])
         if use_mask:
             atanh_mask.requires_grad_()
             parameters.append(atanh_mask)
@@ -275,7 +274,7 @@ class ABS(BackdoorDefense):
             channel_num: int = cur_layer_output.shape[1]  # channels
 
             repeat_shape = [channel_num, self.n_samples]
-            repeat_shape.extend([1] * len(cur_layer_output.shape))
+            repeat_shape.extend([1] * cur_layer_output.dim())
             h_t: torch.Tensor = cur_layer_output.repeat(repeat_shape)
             # (C, n_samples, batch_size, C, H, W)
 
@@ -283,7 +282,7 @@ class ABS(BackdoorDefense):
             if not self.same_range:
                 maxes = cur_layer_output.max()
                 vs *= float(maxes) / self.n_samples
-            vs_shape = [1] * len(cur_layer_output.shape)
+            vs_shape = [1] * cur_layer_output.dim()
             vs_shape[0] = -1
             vs = vs.view(vs_shape)
             # (n_samples, 1, 1, 1)
@@ -357,13 +356,13 @@ class ABS(BackdoorDefense):
     # ---------------------------------- Utils ------------------------------- #
     # Unused
     def filter_img(self):
-        h, w = self.dataset.n_dim
+        h, w = self.dataset.data_shape[1:]
         mask = torch.zeros(h, w, dtype=torch.float)
         mask[2:7, 2:7] = 1
         return to_tensor(mask, non_blocking=False)
 
     def nc_filter_img(self) -> torch.Tensor:
-        h, w = self.dataset.n_dim
+        h, w = self.dataset.data_shape[1:]
         mask = torch.ones(h, w, dtype=torch.float)
         return to_tensor(mask, non_blocking=False)
         # TODO: fix
@@ -374,10 +373,9 @@ class ABS(BackdoorDefense):
         #     mask.add_(1)
 
     def load(self, path: str = None):
-
         if path is None:
-            path = self.folder_path + self.get_filename(target_class=self.target_class) + '_best.npy'
-        _dict = np.load(path, allow_pickle=True).item()
+            path = os.path.join(self.folder_path, self.get_filename(target_class=self.target_class) + '_best.npy')
+        _dict: dict[str, dict[str, torch.Tensor]] = np.load(path, allow_pickle=True).item()
         self.attack.mark.mark = to_tensor(_dict[self.target_class]['mark'])
         self.attack.mark.alpha_mask = to_tensor(_dict[self.target_class]['mask'])
         self.attack.mark.mask = torch.ones_like(self.attack.mark.mark, dtype=torch.bool)
