@@ -1,5 +1,4 @@
 #!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 
 from trojanzoo.defenses import Defense
 from trojanvision.datasets import ImageSet
@@ -21,10 +20,10 @@ class BackdoorDefense(Defense):
 
     def __init__(self, original: bool = False, **kwargs):
         super().__init__(**kwargs)
+        self.dataset: ImageSet
+        self.model: ImageModel
+        self.attack: BadNet  # for linting purpose
         self.original: bool = original
-        self.dataset: ImageSet = self.dataset
-        self.model: ImageModel = self.model
-        self.attack: BadNet = self.attack  # for linting purpose
         self.target_class = self.attack.target_class
 
     def detect(self, **kwargs):
@@ -32,7 +31,7 @@ class BackdoorDefense(Defense):
             self.attack.load(**kwargs)
         if self.attack.name == 'trojannet':
             self.model = self.attack.combined_model
-        self.attack.validate_func()
+        self.attack.validate_fn()
 
     def get_filename(self, **kwargs):
         return self.attack.name + '_' + self.attack.get_filename(**kwargs)
