@@ -1,6 +1,4 @@
-
 #!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 
 from trojanvision.configs import Config, config
 from trojanvision.datasets import ImageSet
@@ -27,15 +25,15 @@ def add_argument(parser: argparse.ArgumentParser):
     return Watermark.add_argument(group)
 
 
-def create(data_shape=None, dataset_name: str = None, dataset: ImageSet = None,
+def create(mark_path: str = None, data_shape: list[int] = None, dataset_name: str = None, dataset: ImageSet = None,
            config: Config = config, **kwargs):
     if data_shape is None:
         assert isinstance(dataset, ImageSet)
-        data_shape: list = [dataset.n_channel]
-        data_shape.extend(dataset.n_dim)
+        data_shape = dataset.data_shape
     if dataset_name is None and dataset is not None:
         dataset_name = dataset.name
     result = config.get_config(dataset_name=dataset_name)['mark']._update(kwargs)
+    result.update(mark_path=mark_path)
     return Watermark(data_shape=data_shape, **result)
 
 
@@ -66,9 +64,9 @@ class Watermark:
                            help='Distributed Mark.')
         return group
 
-    def __init__(self, data_shape: list[int], edge_color: Union[str, torch.Tensor] = 'auto',
-                 mark_path: str = 'trojanzoo/data/mark/square_white.png', mark_alpha: float = 0.0,
-                 mark_height: int = None, mark_width: int = None,
+    def __init__(self, mark_path: str = 'trojanzoo/data/mark/square_white.png',
+                 data_shape: list[int] = None, edge_color: Union[str, torch.Tensor] = 'auto',
+                 mark_alpha: float = 0.0, mark_height: int = None, mark_width: int = None,
                  height_offset: int = 0, width_offset: int = 0,
                  random_pos=False, random_init=False, mark_distributed=False,
                  add_mark_fn=None, **kwargs):
@@ -118,7 +116,7 @@ class Watermark:
     # add mark to the Image with mask.
 
     def add_mark(self, _input: torch.Tensor, random_pos=None, alpha: float = None, **kwargs) -> torch.Tensor:
-        if self.add_mark_fn is not None:
+        if callable(self.add_mark_fn):
             return self.add_mark_fn(_input, random_pos=random_pos, alpha=alpha, **kwargs)
         if random_pos is None:
             random_pos = self.random_pos
@@ -161,7 +159,7 @@ class Watermark:
                 raise ValueError(edge_color)
         else:
             t = torch.as_tensor(edge_color)
-            assert len(t.shape) == 1
+            assert t.dim() == 1
             assert t.shape[0] == data_shape[0]
         return t
 
