@@ -1,5 +1,4 @@
 #!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 
 from trojanvision.datasets import ImageSet
 from trojanzoo.models import _Model, Model
@@ -7,12 +6,23 @@ from trojanvision.environ import env
 from trojanvision.utils import apply_cmap
 
 import torch
-import torch.autograd
 import torch.nn.functional as F
 import re
-import argparse
+import functools
 
+
+from typing import TYPE_CHECKING
+from torch.utils.tensorboard import SummaryWriter
+from torch.optim.optimizer import Optimizer
+from torch.optim.lr_scheduler import _LRScheduler
+import argparse
 from matplotlib.colors import Colormap
+from collections.abc import Callable
+if TYPE_CHECKING:
+    import torch.autograd
+    import torch.cuda.amp
+    import torch.utils.data
+
 from matplotlib.cm import get_cmap
 jet = get_cmap('jet')
 
@@ -164,8 +174,19 @@ class ImageModel(Model):
         self.param_list['imagemodel'] = ['layer', 'width_factor', 'sgm']
         if sgm:
             self.param_list['imagemodel'].extend(['sgm_gamma'])
-        self._model: _ImageModel = self._model
-        self.dataset: ImageSet = self.dataset
+        self._model: _ImageModel
+        self.dataset: ImageSet
+        self.pgd = None  # TODO: python 3.10 type annotation
+
+    def get_data(self, data: tuple[torch.Tensor, torch.Tensor], adv: bool = False, **kwargs) -> tuple[torch.Tensor, torch.Tensor]:
+        if adv and self.pgd is not None:
+            _input, _label = super().get_data(data, **kwargs)
+
+            def loss_fn_new(X: torch.FloatTensor) -> torch.Tensor:  # TODO: use functools.partial
+                return -self.loss(X, _label)
+            adv_x, _ = self.pgd.optimize(_input=_input, loss_fn=loss_fn_new)
+            return adv_x, _label
+        return super().get_data(data, **kwargs)
 
     def get_layer(self, x: torch.Tensor, layer_output: str = 'logits', layer_input: str = 'input') -> torch.Tensor:
         return self._model.get_layer(x, layer_output=layer_output, layer_input=layer_input)
@@ -180,7 +201,7 @@ class ImageModel(Model):
     # Reference: https://keras.io/examples/vision/grad_cam/
     def get_heatmap(self, _input: torch.Tensor, _label: torch.Tensor, method: str = 'grad_cam', cmap: Colormap = jet) -> torch.Tensor:
         squeeze_flag = False
-        if len(_input.shape) == 3:
+        if _input.dim() == 3:
             _input = _input.unsqueeze(0)    # (N, C, H, W)
             squeeze_flag = True
         if isinstance(_label, int):
@@ -227,3 +248,74 @@ class ImageModel(Model):
         if width_factor is not None:
             name += f'x{width_factor:d}'
         return name, layer, width_factor
+
+    def _train(self, epoch: int, optimizer: Optimizer, lr_scheduler: _LRScheduler = None,
+               print_prefix: str = 'Epoch', start_epoch: int = 0,
+               validate_interval: int = 10, save: bool = False, amp: bool = False,
+               loader_train: torch.utils.data.DataLoader = None, loader_valid: torch.utils.data.DataLoader = None,
+               epoch_fn: Callable[..., None] = None,
+               get_data_fn: Callable[..., tuple[torch.Tensor, torch.Tensor]] = None,
+               loss_fn: Callable[..., torch.Tensor] = None,
+               after_loss_fn: Callable[..., None] = None,
+               validate_fn: Callable[..., tuple[float, float]] = None,
+               save_fn: Callable[..., None] = None, file_path: str = None, folder_path: str = None, suffix: str = None,
+               writer: SummaryWriter = None, main_tag: str = 'train', tag: str = '',
+               verbose: bool = True, indent: int = 0,
+               adv_train: bool = False, adv_train_alpha: float = 2.0 / 255, adv_train_epsilon: float = 8.0 / 255,
+               adv_train_valid_epsilon: float = 8.0 / 255,
+               adv_train_iter: int = 7, **kwargs):
+        if adv_train:
+            after_loss_fn_old = after_loss_fn
+            if not callable(after_loss_fn) and hasattr(self, 'after_loss_fn'):
+                after_loss_fn_old = getattr(self, 'after_loss_fn')
+            validate_fn_old = validate_fn if callable(validate_fn) else self._validate
+            loss_fn = loss_fn if callable(loss_fn) else self.loss
+            from trojanvision.optim import PGD  # TODO: consider to move import sentences to top of file
+            self.pgd = PGD(alpha=adv_train_alpha, epsilon=adv_train_valid_epsilon,
+                           iteration=adv_train_iter, stop_threshold=None)
+
+            def after_loss_fn_new(_input: torch.Tensor, _label: torch.Tensor, _output: torch.Tensor,
+                                  loss: torch.Tensor, optimizer: Optimizer, loss_fn: Callable[..., torch.Tensor] = None,
+                                  amp: bool = False, scaler: torch.cuda.amp.GradScaler = None, **kwargs):
+                noise = torch.zeros_like(_input)
+
+                def loss_fn_new(X: torch.FloatTensor) -> torch.Tensor:
+                    return -loss_fn(X, _label)
+                for m in range(self.pgd.iteration):
+                    if amp:
+                        scaler.step(optimizer)
+                        scaler.update()
+                    else:
+                        optimizer.step()
+                    self.eval()
+                    adv_x, _ = self.pgd.optimize(_input=_input, noise=noise, loss_fn=loss_fn_new,
+                                                 iteration=1, epsilon=adv_train_epsilon)
+                    self.train()
+                    loss = loss_fn(adv_x, _label)
+                    if callable(after_loss_fn_old):
+                        after_loss_fn_old(_input=_input, _label=_label, _output=_output,
+                                          loss=loss, optimizer=optimizer, loss_fn=loss_fn,
+                                          amp=amp, scaler=scaler, **kwargs)
+                    if amp:
+                        scaler.scale(loss).backward()
+                    else:
+                        loss.backward()
+
+            def validate_fn_new(get_data_fn: Callable[..., tuple[torch.Tensor, torch.Tensor]] = None,
+                                print_prefix: str = 'Validate', **kwargs) -> tuple[float, float]:
+                _, clean_acc = validate_fn_old(print_prefix='Validate Clean', main_tag='valid clean',
+                                               get_data_fn=None, **kwargs)
+                _, adv_acc = validate_fn_old(print_prefix='Validate Adv', main_tag='valid adv',
+                                             get_data_fn=functools.partial(get_data_fn, adv=True), **kwargs)
+                return adv_acc, clean_acc
+
+            after_loss_fn = after_loss_fn_new
+            validate_fn = validate_fn_new
+
+        super()._train(epoch=epoch, optimizer=optimizer, lr_scheduler=lr_scheduler,
+                       print_prefix=print_prefix, start_epoch=start_epoch,
+                       validate_interval=validate_interval, save=save, amp=amp,
+                       loader_train=loader_train, loader_valid=loader_valid,
+                       epoch_fn=epoch_fn, get_data_fn=get_data_fn, loss_fn=loss_fn, after_loss_fn=after_loss_fn, validate_fn=validate_fn,
+                       save_fn=save_fn, file_path=file_path, folder_path=folder_path, suffix=suffix,
+                       writer=writer, main_tag=main_tag, tag=tag, verbose=verbose, indent=indent, **kwargs)
