@@ -1,5 +1,4 @@
 #!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 
 from trojanzoo.configs import config, Config
 from trojanzoo.environ import env
@@ -8,14 +7,17 @@ from trojanzoo.utils.data import dataset_to_list
 from trojanzoo.utils.output import ansi, prints, Indent_Redirect
 
 import torch
-import torch.cuda
-import torch.utils.data
 from torchvision import transforms
 import numpy as np
-import argparse
 import os
 import sys
-from typing import Any, Union
+
+from typing import TYPE_CHECKING
+from typing import Union    # TODO: python 3.10
+import argparse    # TODO: python 3.10
+if TYPE_CHECKING:
+    import torch.cuda
+    import torch.utils.data
 
 
 redirect = Indent_Redirect(buffer=True, indent=0)
@@ -188,7 +190,7 @@ class Dataset:
         assert (length is None) != (percent is None)  # XOR check
         length = length if length is not None else int(len(dataset) * percent)
         indices = np.arange(len(dataset))
-        np.random.shuffle(indices)
+        np.random.shuffle(indices)  # TODO: random seed?
         if isinstance(dataset, torch.utils.data.Subset):
             idx = np.array(dataset.indices)
             indices = idx[indices]
