@@ -1,5 +1,4 @@
 #!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 
 from trojanzoo.configs import config, Config
 from trojanzoo.datasets import Dataset
@@ -9,9 +8,13 @@ from trojanzoo.utils import get_name
 from trojanzoo.utils.output import ansi
 from trojanzoo.utils.process import Model_Process
 
-import argparse
 import os
-from typing import Union
+
+from typing import TYPE_CHECKING
+from typing import Union    # TODO: python 3.10
+import argparse    # TODO: python 3.10
+if TYPE_CHECKING:
+    pass
 
 
 class Defense(Model_Process):
