@@ -1,31 +1,39 @@
 #!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 
 
-from trojanzoo.configs import config, Config
+from trojanzoo.configs import config
 from trojanzoo.datasets import Dataset
 from trojanzoo.environ import env
 from trojanzoo.utils import add_noise, empty_cache, repeat_to_batch, to_tensor
 from trojanzoo.utils import get_name
-from trojanzoo.utils.output import ansi, get_ansi_len, prints, output_iter
 from trojanzoo.utils.logger import MetricLogger, SmoothedValue
+from trojanzoo.utils.output import ansi, get_ansi_len, prints, output_iter
+from trojanzoo.utils.tensor import onehot_label
 
 
 import torch
 import torch.nn as nn
-import torch.optim
-import torch.utils.data
-import torch.cuda.amp
-from torch.optim.optimizer import Optimizer
-from torch.optim.lr_scheduler import _LRScheduler
+import torch.nn.functional as F
 from torch.utils.tensorboard import SummaryWriter
 import numpy as np
-import argparse
+import math
 import os
 from tqdm import tqdm
 from collections import OrderedDict
-from collections.abc import Callable, Iterable    # TODO: callable (many places) (wait for python update)
-from typing import Generator, Iterator, Mapping, Set, Union, Optional
+from collections.abc import Iterable    # TODO: callable (many places) (wait for python update)
+from typing import Generator, Iterator, Mapping, Set
+
+from typing import TYPE_CHECKING
+from typing import Union, Optional    # TODO: python 3.10
+from trojanzoo.configs import Config    # TODO: python 3.10
+from torch.optim.optimizer import Optimizer
+from torch.optim.lr_scheduler import _LRScheduler
+import argparse
+from collections.abc import Callable
+if TYPE_CHECKING:
+    import torch.optim
+    import torch.cuda.amp
+    import torch.utils.data
 
 # redirect = Indent_Redirect(buffer=True, indent=0)
 
@@ -244,10 +252,21 @@ class Model:
 
     # define loss function
     # Cross Entropy
-    def define_criterion(self, **kwargs) -> nn.CrossEntropyLoss:    # TODO: linting, or maybe nn.Module for generic?
+    # TODO: linting, or maybe nn.Module for generic?
+    def define_criterion(self, loss_type='ce', num_classes: int = None, **kwargs) -> nn.CrossEntropyLoss:
         if 'weight' not in kwargs.keys():
             kwargs['weight'] = self.loss_weights
-        return nn.CrossEntropyLoss(**kwargs)
+        criterion = nn.CrossEntropyLoss(**kwargs)
+        if loss_type == 'jsd':
+            num_classes = num_classes if num_classes is not None else self.num_classes
+
+            def jsd(_output: torch.Tensor, _label: torch.Tensor, **kwargs):
+                p = onehot_label(_label, num_classes)
+                q: torch.Tensor = self.softmax(_output)
+                log_q = F.log_softmax(_output)
+                sum_pq = p + q
+                loss = sum_pq * (sum_pq.log() - math.log(2))
+        return criterion
 
     # -----------------------------Load & Save Model------------------------------------------- #
 
@@ -330,40 +349,43 @@ class Model:
                print_prefix: str = 'Epoch', start_epoch: int = 0,
                validate_interval: int = 10, save: bool = False, amp: bool = False,
                loader_train: torch.utils.data.DataLoader = None, loader_valid: torch.utils.data.DataLoader = None,
+               epoch_fn: Callable[..., None] = None,
                get_data_fn: Callable[..., tuple[torch.Tensor, torch.Tensor]] = None,
-               loss_fn: Callable[[torch.Tensor, torch.Tensor, torch.Tensor], torch.Tensor] = None,
-               after_loss_fn: Callable[..., torch.Tensor] = None,
-               validate_func: Callable[..., tuple[float, ...]] = None, epoch_func: Callable[..., None] = None,
-               save_fn: Callable = None, file_path: str = None, folder_path: str = None, suffix: str = None,
+               loss_fn: Callable[..., torch.Tensor] = None,
+               after_loss_fn: Callable[..., None] = None,
+               validate_fn: Callable[..., tuple[float, float]] = None,
+               save_fn: Callable[..., None] = None, file_path: str = None, folder_path: str = None, suffix: str = None,
                writer: SummaryWriter = None, main_tag: str = 'train', tag: str = '',
                verbose: bool = True, indent: int = 0, **kwargs):
         loader_train = loader_train if loader_train is not None else self.dataset.loader['train']
-        get_data_fn = get_data_fn if get_data_fn is not None else self.get_data
-        loss_fn = loss_fn if loss_fn is not None else self.loss
-        validate_func = validate_func if validate_func is not None else self._validate
-        save_fn = save_fn if save_fn is not None else self.save
-        if after_loss_fn is None and hasattr(self, 'after_loss_fn'):
-            after_loss_fn = self.after_loss_fn
-        if epoch_func is None and hasattr(self, 'epoch_func'):
-            epoch_func = self.epoch_func
+        get_data_fn = get_data_fn if callable(get_data_fn) else self.get_data
+        loss_fn = loss_fn if callable(loss_fn) else self.loss
+        validate_fn = validate_fn if callable(validate_fn) else self._validate
+        save_fn = save_fn if callable(save_fn) else self.save
+        # if not callable(iter_fn) and hasattr(self, 'iter_fn'):
+        #     iter_fn = getattr(self, 'iter_fn')
+        if not callable(epoch_fn) and hasattr(self, 'epoch_fn'):
+            epoch_fn = getattr(self, 'epoch_fn')
+        if not callable(after_loss_fn) and hasattr(self, 'after_loss_fn'):
+            after_loss_fn = getattr(self, 'after_loss_fn')
 
         scaler: torch.cuda.amp.GradScaler = None
         if not env['num_gpus']:
             amp = False
         if amp:
             scaler = torch.cuda.amp.GradScaler()
-        _, best_acc = validate_func(loader=loader_valid, get_data_fn=get_data_fn, loss_fn=loss_fn,
-                                    writer=writer, tag=tag, _epoch=start_epoch,
-                                    verbose=verbose, indent=indent, **kwargs)
+        _, best_acc = validate_fn(loader=loader_valid, get_data_fn=get_data_fn, loss_fn=loss_fn,
+                                  writer=None, tag=tag, _epoch=start_epoch,
+                                  verbose=verbose, indent=indent, **kwargs)
 
         params: list[list[nn.Parameter]] = [param_group['params'] for param_group in optimizer.param_groups]
         total_iter = epoch * len(loader_train)
         for _epoch in range(epoch):
-            _epoch += + 1
-            if callable(epoch_func):
+            _epoch += 1
+            if callable(epoch_fn):
                 self.activate_params([])
-                epoch_func(optimizer=optimizer, lr_scheduler=lr_scheduler,
-                           _epoch=_epoch, epoch=epoch, start_epoch=start_epoch)
+                epoch_fn(optimizer=optimizer, lr_scheduler=lr_scheduler,
+                         _epoch=_epoch, epoch=epoch, start_epoch=start_epoch)
                 self.activate_params(params)
             logger = MetricLogger()
             logger.meters['loss'] = SmoothedValue()
@@ -389,12 +411,19 @@ class Model:
                 loss = loss_fn(_input, _label, _output=_output)
                 if amp:
                     scaler.scale(loss).backward()
+                    if callable(after_loss_fn):
+                        after_loss_fn(_input=_input, _label=_label, _output=_output,
+                                      loss=loss, optimizer=optimizer, loss_fn=loss_fn,
+                                      amp=amp, scaler=scaler,
+                                      _iter=_iter, total_iter=total_iter)
                     scaler.step(optimizer)
                     scaler.update()
                 else:
                     loss.backward()
                     if callable(after_loss_fn):
-                        after_loss_fn(optimizer=optimizer,
+                        after_loss_fn(_input=_input, _label=_label, _output=_output,
+                                      loss=loss, optimizer=optimizer, loss_fn=loss_fn,
+                                      amp=amp, scaler=scaler,
                                       _iter=_iter, total_iter=total_iter)
                         # start_epoch=start_epoch, _epoch=_epoch, epoch=epoch)
                     optimizer.step()
@@ -417,9 +446,9 @@ class Model:
                 lr_scheduler.step()
             if validate_interval != 0:
                 if _epoch % validate_interval == 0 or _epoch == epoch:
-                    _, cur_acc = validate_func(loader=loader_valid, get_data_fn=get_data_fn, loss_fn=loss_fn,
-                                               writer=writer, tag=tag, _epoch=_epoch + start_epoch,
-                                               verbose=verbose, indent=indent, **kwargs)
+                    _, cur_acc = validate_fn(loader=loader_valid, get_data_fn=get_data_fn, loss_fn=loss_fn,
+                                             writer=writer, tag=tag, _epoch=_epoch + start_epoch,
+                                             verbose=verbose, indent=indent, **kwargs)
                     if cur_acc >= best_acc:
                         if verbose:
                             prints('{green}best result update!{reset}'.format(**ansi), indent=indent)
@@ -434,7 +463,7 @@ class Model:
     def _validate(self, full=True, print_prefix='Validate', indent=0, verbose=True,
                   loader: torch.utils.data.DataLoader = None,
                   get_data_fn: Callable[..., tuple[torch.Tensor, torch.Tensor]] = None,
-                  loss_fn: Callable[[torch.Tensor, torch.Tensor, torch.Tensor], torch.Tensor] = None,
+                  loss_fn: Callable[..., torch.Tensor] = None,
                   writer: SummaryWriter = None, main_tag: str = 'valid', tag: str = '', _epoch: int = None,
                   **kwargs) -> tuple[float, float]:
         self.eval()
@@ -454,9 +483,9 @@ class Model:
                 header = '{upline}{clear_line}'.format(**ansi) + header
                 loader_epoch = tqdm(loader_epoch)
             loader_epoch = logger.log_every(loader_epoch, header=header, indent=indent)
-        with torch.no_grad():
-            for data in loader_epoch:
-                _input, _label = get_data_fn(data, mode='valid', **kwargs)
+        for data in loader_epoch:
+            _input, _label = get_data_fn(data, mode='valid', **kwargs)
+            with torch.no_grad():
                 _output = self(_input)
                 loss = float(loss_fn(_input, _label, _output=_output, **kwargs))
                 acc1, acc5 = self.accuracy(_output, _label, topk=(1, 5))
@@ -465,14 +494,14 @@ class Model:
                 logger.meters['top1'].update(acc1, batch_size)
                 logger.meters['top5'].update(acc5, batch_size)
         loss, acc = logger.meters['loss'].global_avg, logger.meters['top1'].global_avg
-        if isinstance(writer, SummaryWriter) and isinstance(_epoch, int):
+        if isinstance(writer, SummaryWriter) and isinstance(_epoch, int) and main_tag:
             writer.add_scalars(main_tag='Loss/' + main_tag, tag_scalar_dict={tag: loss}, global_step=_epoch)
             writer.add_scalars(main_tag='Acc/' + main_tag, tag_scalar_dict={tag: acc}, global_step=_epoch)
         return loss, acc
 
     # -------------------------------------------Utility--------------------------------------- #
 
-    def get_data(self, data: tuple[torch.Tensor, torch.Tensor], **kwargs):
+    def get_data(self, data: tuple[torch.Tensor, torch.Tensor], **kwargs) -> tuple[torch.Tensor, torch.Tensor]:
         if self.dataset is not None:
             return self.dataset.get_data(data, **kwargs)
         else:
@@ -492,7 +521,7 @@ class Model:
                 if k > self.num_classes:
                     res.append(100.0)
                 else:
-                    correct_k = correct[:k].flatten().sum(dtype=torch.float32)
+                    correct_k = float(correct[:k].sum(dtype=torch.float32))
                     res.append(correct_k * (100.0 / batch_size))
             return res
 
