@@ -1,12 +1,15 @@
 #!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 
 from trojanzoo.utils import output_memory
 from trojanzoo.utils.process import Process
 from trojanzoo.utils.output import prints
 
 import torch
-from collections.abc import Callable
+
+from typing import TYPE_CHECKING
+from collections.abc import Callable    # TODO: python 3.10
+if TYPE_CHECKING:
+    pass
 
 
 class Optimizer(Process):
